@@ -4,7 +4,7 @@
     begin                : Sat Aug 10 2002
     copyright            : (C) 2002 by Richard Garand
     email                : richard@garandnet.net
-    $Id: account.cpp,v 1.14 2002/09/04 07:07:35 richard Exp $
+    $Id: account.cpp,v 1.19 2002/09/13 02:25:16 richard Exp $
  ***************************************************************************/
 
 /***************************************************************************
@@ -20,6 +20,7 @@
  
 #include "account.h"
 #include "kbudgetdoc.h"
+#include "kbundoaction.h"
 
 Account::Account()
   : initialBalance(0)
@@ -99,18 +100,21 @@ int Account::id() const
 
 void Account::setType(types tp)
 {
-	m_type = tp;
+  m_type = tp;
 }
 
 Account::types Account::type() const
 {
-	return m_type;
+  return m_type;
 }
 
 void Account::setName(QString n)
 {
-	m_name = n;
-	m_doc->setModified(true);
+  if ( m_doc->canAddUndo() )
+    m_doc->addUndo(new KBAccountUndo(m_doc, KBAccountUndo::UA_RENAME, m_id));
+  m_name = n;
+  m_doc->setModified(true);
+  m_doc->signalAcctChanged(m_id);
 }
 
 QString Account::name() const
@@ -120,61 +124,34 @@ QString Account::name() const
 
 void Account::addTransaction(int id)
 {
-  Transaction &tr = m_doc->getTransaction(id);
-  QDate date = tr.date();
-
-  QMap<int,year_t>::iterator year;
-  year_t::iterator month;
-  createMonth(date.year(), date.month(), year, month);
-  Month& mnth = months[date.year()][date.month()];
-
-  mnth.transactions.push_back(id);
-  float add = tr.value(); // TODO: \n // if it's from, we want to subtract, not add
-  add *= m_id == tr.to() ? 1 : m_id == tr.from() ? -1 : 0;
-//  add *= m_type == AC_INCOME ? 1 : 1;
-  // TODO: clean this up
-  float original = mnth.endBalance;
-  float sum = original + add;
-  mnth.endBalance = sum;
-
-  float budgetChange = add;//tr.value();
-//  budgetChange *= m_type == AC_INCOME ? 1 : m_type == AC_EXPENSE ? -1 : 0;
-  budgetChange *= m_type == AC_EXPENSE ? -1 : 1;
-  mnth.endBudget += budgetChange;
-
-  month++;
-  while ( year != months.end() ) {
-    while ( month != (*year).end() ) {
-      (*month).startBalance += add;
-      (*month).endBalance += add;
-      (*month).startBudget += budgetChange;
-      (*month).endBudget += budgetChange;
-      month++;
-    }
-    year++;
-    if ( year != months.end() )
-      month = (*year).begin();
-  }
-
-  m_doc->signalAcctChanged(id);
+  transactionUpdate(id, true);
 }
 
 void Account::removeTransaction(int id)
 {
-  Transaction &tr = m_doc->getTransaction(id);
-  months[tr.date().year()][tr.date().month()].transactions.remove(id);
-  // TODO: update balances
-  if ( months[tr.date().year()][tr.date().month()].transactions.size() == 0 ) {
-    months[tr.date().year()].remove(tr.date().month());
-    if ( months[tr.date().year()].keys().size() == 0 )
-      months.remove(tr.date().year());
-  }
-  m_doc->signalAcctChanged(id);
+  transactionUpdate(id, false);
 }
 
 QValueList<int> Account::getTransactions(int year, int month) const
 {
-  return QValueList<int>();
+  QValueList<int> list;
+
+  if ( !months.size() )
+    return list;
+
+  QMap<int,year_t>::const_iterator yr;// = months.begin()
+  year_t::const_iterator mn;
+  for ( yr = months.begin(); yr != months.end(); yr++ ) {
+    if ( year && yr.key() != year )
+      continue;
+    for ( mn = (*yr).begin(); mn != (*yr).end(); mn++ ) {
+      if ( month && mn.key() != month )
+        continue;
+      list += (*mn).transactions;
+    }
+  }
+
+  return list;
 }
 
 QValueList<int> Account::getTransactions(QDate start, QDate end) const
@@ -197,15 +174,16 @@ float Account::startingBalance() const
 
 void Account::setStartingBalance(float balance)
 {
+  if ( m_doc->canAddUndo() )
+    m_doc->addUndo(new KBAccountUndo(m_doc, KBAccountUndo::UA_STBALANCE, m_id));
+
   float diff = balance - initialBalance;
   initialBalance = balance;
-  for ( QMap<int,year_t>::iterator yr = months.begin(); yr != months.end(); yr++ ) {
-    for ( year_t::iterator month = (*yr).begin(); month != (*yr).end(); month++ ) {
-      (*month).startBalance += diff;
-      (*month).endBalance += diff;
-    }
-  }
+  QMap<int,year_t>::iterator yr = months.begin();
+  year_t::iterator month = (*yr).begin();
+  add(yr, month, diff, 0.0f);
   m_doc->setModified(true);
+  m_doc->signalAcctChanged(m_id);
 }
 
 float Account::change() const
@@ -264,17 +242,12 @@ void Account::setBudgeted(QDate month, float value)
   year_t::iterator monthIt;
   createMonth(month.year(), month.month(), yearIt, monthIt);
 
-  float add = value - (*monthIt).budgeted;
+  float addBudget = value - (*monthIt).budgeted;
   (*monthIt).budgeted = value;
-  (*monthIt).endBudget += add;
+  (*monthIt).endBudget += addBudget;
   
   monthIt++;
-  for ( ; yearIt != months.end(); yearIt++ ) {
-    for ( ; monthIt != (*yearIt).end(); monthIt++ ) {
-      (*monthIt).startBudget += add;
-      (*monthIt).endBudget += add;
-    }
-  }
+  add(yearIt, monthIt, 0, addBudget);
 
   m_doc->setModified(true);
 }
@@ -312,7 +285,7 @@ Account Account::clone(QDate fromDate, QDate toDate) const
       newAcct.months[fromDate.year()][fromDate.month()].startBalance = newAcct.initialBalance;
       newAcct.months[fromDate.year()][fromDate.month()].endBalance = newAcct.initialBalance;
     } else
-      newAcct.initialBalance = 0;
+      newAcct.initialBalance = initialBalance;
 
   return newAcct;
 }
@@ -404,3 +377,61 @@ void Account::dump() const
 {
 //  printf( "Account %i: %0.2f to %0.2f\n", m_id, initialBalance, getBalance());
 }
+
+void Account::transactionUpdate(int id, bool adding)
+{
+  Transaction &tr = m_doc->getTransaction(id);
+  QDate date = tr.date();
+
+  QMap<int,year_t>::iterator year;
+  year_t::iterator month;
+  //if ( adding )
+    createMonth(date.year(), date.month(), year, month);
+  Month& mnth = months[date.year()][date.month()];
+
+  float addBalance = tr.value();
+  // if it's from, we want to subtract, not add
+  addBalance *= m_id == tr.to() ? 1 : m_id == tr.from() ? -1 : 0;
+  // if we're removing, negate the value
+  if ( !adding )
+    addBalance *= -1;
+  mnth.endBalance += addBalance;
+
+  float budgetChange = addBalance;
+  budgetChange *= m_type == AC_EXPENSE ? -1 : 1;
+  mnth.endBudget += budgetChange;
+
+  if ( adding )
+    mnth.transactions.push_back(id);
+  else {
+    mnth.transactions.remove(id);
+    // clean up the map
+    // TODO: re-enable this and fix it (see ~/documents/bug.kbudget for procedure)
+/*    if ( !mnth.transactions.size() ) {
+      months[date.year()].remove(date.month());
+      if ( months[date.year()].keys().size() == 0 )
+        months.remove(date.year());
+    }*/
+  }
+
+  month++;
+  add(year, month, addBalance, budgetChange);
+
+  m_doc->signalAcctChanged(m_id);
+}
+
+void Account::add(QMap<int,year_t>::iterator year, year_t::iterator month, float balance, float budget)
+{
+  while ( year != months.end() ) {
+    while ( month != (*year).end() ) {
+      (*month).startBalance += balance;
+      (*month).endBalance += balance;
+      (*month).startBudget += budget;
+      (*month).endBudget += budget;
+      month++;
+    }
+    year++;
+    if ( year != months.end() )
+      month = (*year).begin();
+  }
+}
