@@ -4,7 +4,7 @@
     begin                : Sat Aug 10 16:03:35 CST 2002
     copyright            : (C) 2002 by Richard Garand
     email                : richard@garandnet.net
-    $Id: kbudget.cpp,v 1.15 2002/09/06 23:39:58 richard Exp $
+    $Id: kbudget.cpp,v 1.21 2002/09/11 04:44:56 richard Exp $
  ***************************************************************************/
 
 /***************************************************************************
@@ -30,7 +30,8 @@
 #include <klocale.h>
 #include <kconfig.h>
 #include <kstdaction.h>
-#include <kiconloader.h>
+#include <kkeydialog.h>
+#include <kedittoolbar.h>
 
 // application specific includes
 #include "kbudget.h"
@@ -38,6 +39,7 @@
 #include "kbudgetdoc.h"
 #include "knewaccount.h"
 #include "knewtransaction.h"
+#include "ktransactionwindow.h"
 
 #define ID_STATUS_MSG 1
 
@@ -49,6 +51,7 @@ KBudgetApp::KBudgetApp(QWidget* , const char* name):KMainWindow(0, name)
   // call inits to invoke all other construction parts
   initStatusBar();
   initActions();
+  initPopups();
   initDocument();
   initView();
 	
@@ -66,13 +69,13 @@ KBudgetApp::KBudgetApp(QWidget* , const char* name):KMainWindow(0, name)
   editPaste->setEnabled(false);
   viewYear->setEnabled(false);
   viewSpecial->setEnabled(false);
-  accountDelete->setEnabled(false);
-  accountsRename->setEnabled(false);
+  slotEnableAccountFunctions(false);
+  slotUndoStatus(false, false);
 }
 
 KBudgetApp::~KBudgetApp()
 {
-
+  KTransactionWindow::closeAllWindows();
 }
 
 void KBudgetApp::initActions()
@@ -86,6 +89,8 @@ void KBudgetApp::initActions()
   fileClose = KStdAction::close(this, SLOT(slotFileClose()), actionCollection());
   filePrint = KStdAction::print(this, SLOT(slotFilePrint()), actionCollection());
   fileQuit = KStdAction::quit(this, SLOT(slotFileQuit()), actionCollection());
+  editUndo = KStdAction::undo(this, SLOT(slotEditUndo()), actionCollection());
+  editRedo = KStdAction::redo(this, SLOT(slotEditRedo()), actionCollection());
   editCut = KStdAction::cut(this, SLOT(slotEditCut()), actionCollection());
   editCopy = KStdAction::copy(this, SLOT(slotEditCopy()), actionCollection());
   editPaste = KStdAction::paste(this, SLOT(slotEditPaste()), actionCollection());
@@ -93,11 +98,15 @@ void KBudgetApp::initActions()
   viewMonth = new KAction(i18n("This &Month"), 0, 0, this, SLOT(slotViewMonth()), actionCollection(), "view_month");
   viewSpecial = new KAction(i18n("Select &Period"), 0, 0, this, SLOT(slotViewSpecial()), actionCollection(), "select_period");
   accountsNew = new KAction(i18n("&New Account"), "folder_new", 0, this, SLOT(slotAccountsNew()), actionCollection(), "accounts_new");
-  accountsRename = new KAction(i18n("&Rename account"), "edit", 0, 0, 0, actionCollection(), "accounts_rename");
+  accountsEdit = new KAction(i18n("&Edit Account"), "edit", 0, 0, 0, actionCollection(), "accounts_edit");
+  accountsRename = new KAction(i18n("&Rename Account"), "editclear", 0, 0, 0, actionCollection(), "accounts_rename");
   accountDelete = new KAction(i18n("&Delete Account"), "editdelete", 0, this, SLOT(slotDeleteAccount()), actionCollection(), "accounts_delete");
+  accountsTransactions = new KAction(i18n("&View transactions"), 0, 0, 0, 0, actionCollection(), "accounts_transactions");
   transactionsNew = new KAction(i18n("&New Transaction"), "mail_forward", 0, this, SLOT(slotTransactionsNew()), actionCollection(), "transactions_new");
   viewToolBar = KStdAction::showToolbar(this, SLOT(slotViewToolBar()), actionCollection());
   viewStatusBar = KStdAction::showStatusbar(this, SLOT(slotViewStatusBar()), actionCollection());
+  KStdAction::keyBindings(this, SLOT(slotConfKeys()), actionCollection());
+  KStdAction::configureToolbars(this, SLOT(slotConfToolbar()), actionCollection());
 
   fileNewWindow->setStatusText(i18n("Opens a new application window"));
   fileNew->setStatusText(i18n("Creates a new document"));
@@ -120,6 +129,7 @@ void KBudgetApp::initActions()
   viewToolBar->setStatusText(i18n("Enables/disables the toolbar"));
   viewStatusBar->setStatusText(i18n("Enables/disables the statusbar"));
   accountDelete->setStatusText(i18n("Deletes the currently selected account"));
+  accountsTransactions->setStatusText(i18n("Opens a new window to display the account's transactions"));
 
   // use the absolute path to your kbudgetui.rc file for testing purpose in createGUI();
   createGUI();
@@ -139,6 +149,7 @@ void KBudgetApp::initDocument()
 {
   doc = new KBudgetDoc(this);
   connect(doc, SIGNAL(saveableStatus(bool)), SLOT(slotEnableSave(bool)));
+  connect(doc, SIGNAL(undoStatus(bool,bool)), SLOT(slotUndoStatus(bool,bool)));
   doc->newDocument();
 }
 
@@ -150,13 +161,22 @@ void KBudgetApp::initView()
 
   view = new KBudgetView(this);
   connect(view, SIGNAL(accountSelectionStatus(bool)), SLOT(slotEnableAccountFunctions(bool)));
+  connect(view, SIGNAL(contextMenuRequested(const QPoint&)), SLOT(slotMainContextMenu(const QPoint&)));
+  connect(accountsEdit, SIGNAL(activated()), view, SLOT(slotEditAccount()));
   connect(accountsRename, SIGNAL(activated()), view, SLOT(slotRenameAccount()));
+  connect(accountsTransactions, SIGNAL(activated()), view, SLOT(slotTransactions()));
   doc->addView(view);
   setCentralWidget(view);	
   setCaption(doc->URL().fileName(),false);
 
 }
 
+void KBudgetApp::initPopups()
+{
+  mainPopup = static_cast<QPopupMenu*>(factory()->container("account_view_popup", this));
+  if (!mainPopup) mainPopup = new QPopupMenu(this);
+}
+
 void KBudgetApp::openDocumentFile(const KURL& url)
 {
   slotStatusMsg(i18n("Opening file..."));
@@ -337,6 +357,7 @@ void KBudgetApp::slotFileOpen()
 
 void KBudgetApp::slotFileOpenRecent(const KURL& url)
 {
+  // TODO: fix fileOpenRecent bug
   slotStatusMsg(i18n("Opening file..."));
 	
   if(!doc->saveModified())
@@ -414,7 +435,7 @@ void KBudgetApp::slotFileQuit()
   saveOptions();
   // close the first window, the list makes the next one the first again.
   // This ensures that queryClose() is called on each window to ask for closing
-  KMainWindow* w;
+//  KMainWindow* w;
   if(memberList)
   {
     for ( unsigned int i= 0; i< memberList->count(); i++ )
@@ -427,6 +448,16 @@ void KBudgetApp::slotFileQuit()
   }
 }
 
+void KBudgetApp::slotEditUndo()
+{
+  doc->undo();
+}
+
+void KBudgetApp::slotEditRedo()
+{
+  doc->redo();
+}
+
 void KBudgetApp::slotEditCut()
 {
   slotStatusMsg(i18n("Cutting selection..."));
@@ -475,14 +506,14 @@ void KBudgetApp::slotViewSpecial()
 void KBudgetApp::slotAccountsNew()
 {
   slotStatusMsg(i18n("Creating a new account..."));
-  (new KNewAccount(this))->show();
+  (new KNewAccount(this, doc))->show();
   slotStatusMsg(i18n("Ready."));
 }
 
 void KBudgetApp::slotTransactionsNew()
 {
   slotStatusMsg(i18n("Creating a new transaction..."));
-  (new KNewTransaction(this))->show();
+  (new KNewTransaction(this, doc))->show();
   slotStatusMsg(i18n("Ready."));
 }
 
@@ -520,6 +551,18 @@ void KBudgetApp::slotViewStatusBar()
   slotStatusMsg(i18n("Ready."));
 }
 
+void KBudgetApp::slotConfKeys()
+{
+  KKeyDialog::configureKeys(actionCollection(), xmlFile(), true, this);
+}
+
+void KBudgetApp::slotConfToolbar()
+{
+  KEditToolbar *dlg = new KEditToolbar(guiFactory(),this);
+  if (dlg->exec())
+    createGUI();
+  delete dlg;
+}
 
 void KBudgetApp::slotStatusMsg(const QString &text)
 {
@@ -542,5 +585,18 @@ void KBudgetApp::slotDeleteAccount()
 void KBudgetApp::slotEnableAccountFunctions(bool enabled)
 {
   accountDelete->setEnabled(enabled);
+  accountsEdit->setEnabled(enabled);
   accountsRename->setEnabled(enabled);
+  accountsTransactions->setEnabled(enabled);
+}
+
+void KBudgetApp::slotMainContextMenu(const QPoint &p)
+{
+  mainPopup->popup(p);
+}
+
+void KBudgetApp::slotUndoStatus(bool canUndo, bool canRedo)
+{
+  editUndo->setEnabled(canUndo);
+  editRedo->setEnabled(canRedo);
 }
