@@ -4,7 +4,7 @@
     begin                : Thu Aug 29 2002
     copyright            : (C) 2002 by Richard Garand
     email                : richard@garandnet.net
-    $Id: knewaccount.cpp,v 1.2 2002/09/04 02:04:16 richard Exp $
+    $Id: knewaccount.cpp,v 1.5 2002/09/13 03:32:26 richard Exp $
  ***************************************************************************/
 
 /***************************************************************************
@@ -20,13 +20,14 @@
 #include <qlabel.h>
 #include <klocale.h>
 #include <kmessagebox.h>
+#include <kmainwindow.h>
  
 #include "knewaccount.h"
 #include "kbudget.h"
 #include "kbudgetdoc.h"
 
-KNewAccount::KNewAccount(KBudgetApp* parent)
-  : KDialogBase(parent, "knewaccount", false, i18n("New Account"), Ok|Cancel), mainWin(parent)
+KNewAccount::KNewAccount(KMainWindow* parent, KBudgetDoc* d, int id)
+  : KDialogBase(parent, "knewaccount", false, i18n("New Account"), Ok|Cancel), doc(d), account(id)
 {
   QWidget* page = new QWidget(this);
   setMainWidget(page);
@@ -52,6 +53,15 @@ KNewAccount::KNewAccount(KBudgetApp* parent)
   type->insertItem(i18n("Cash"));
 
   initialBalance->setPrecision(2);
+
+  if ( id >= 0 ) {
+    Account& acct = doc->getAccount(id);
+    name->setText(acct.name());
+    type->setCurrentItem((int)acct.type());
+    type->setEnabled(false);
+    initialBalance->setValue(acct.startingBalance());
+    setCaption(i18n("Edit Account"));
+  }
 }
 
 KNewAccount::~KNewAccount()
@@ -65,10 +75,17 @@ void KNewAccount::slotOk()
   float balance = (float)initialBalance->value();
 
   if ( !acctName.stripWhiteSpace().length() )
-    KMessageBox::sorry(0, i18n("You must enter an account name to create an account."));
+    KMessageBox::sorry(0, i18n("You must enter an account name"));
   else {
-    KBudgetDoc* doc = mainWin->getDocument();
-    doc->newAccount((Account::types)acctType, acctName, balance);
+    doc->beginUndoGroup();
+    if ( account < 0 )
+      doc->newAccount((Account::types)acctType, acctName, balance);
+    else {
+      Account& acct = doc->getAccount(account);
+      acct.setName(acctName);
+      acct.setStartingBalance(balance);
+    }
+    doc->endUndoGroup();
     KDialogBase::slotOk();
   }
 }
