@@ -4,7 +4,7 @@
     begin                : Fri Aug 30 2002
     copyright            : (C) 2002 by Richard Garand
     email                : richard@garandnet.net
-    $Id: knewtransaction.h,v 1.2 2002/09/04 02:04:16 richard Exp $
+    $Id: knewtransaction.h,v 1.3 2002/09/10 01:02:50 richard Exp $
  ***************************************************************************/
 
 /***************************************************************************
@@ -27,7 +27,8 @@
 #include <knuminput.h>
 #include <kdateedit.h>
 
-class KBudgetApp;
+class KMainWindow;
+class KBudgetDoc;
 class Account;
 
 /**Creates and handles a new transaction dialog box
@@ -36,13 +37,16 @@ class Account;
 
 class KNewTransaction : public KDialogBase  {
   public: 
-	KNewTransaction(KBudgetApp* parent);
+	KNewTransaction(KMainWindow* parent, KBudgetDoc* doc, int transaction = -1);
 	~KNewTransaction();
   private:
     /** OK button clicked */
     void slotOk();
   
-    const KBudgetApp* mainWin;
+    const KMainWindow* mainWin;
+    KBudgetDoc* document;
+    int id;
+  
     KDateEdit* date;
     KComboBox* from;
     KComboBox* to;
